@@ -46,6 +46,10 @@ type ReadSettings struct {
 	// false.
 	PreserveCData bool
 
+	// When an element has two or more attributes with the same name,
+	// preserve them instead of keeping only one. Default: false.
+	PreserveDuplicateAttrs bool
+
 	// Entity to be passed to standard xml.Decoder. Default: nil.
 	Entity map[string]string
 }
@@ -549,6 +553,15 @@ func (e *Element) name() string {
 	return e.Tag
 }
 
+// ReindexChildren recalculates the index values of the element's child
+// tokens. This is necessary only if you have manually manipulated the
+// element's `Child` array.
+func (e *Element) ReindexChildren() {
+	for i := 0; i < len(e.Child); i++ {
+		e.Child[i].setIndex(i)
+	}
+}
+
 // Text returns all character data immediately following the element's opening
 // tag.
 func (e *Element) Text() string {
@@ -827,7 +840,7 @@ func (e *Element) readFrom(ri io.Reader, settings ReadSettings) (n int64, err er
 		case xml.StartElement:
 			e := newElement(t.Name.Space, t.Name.Local, top)
 			for _, a := range t.Attr {
-				e.createAttr(a.Name.Space, a.Name.Local, a.Value, e)
+				e.createAttr(a.Name.Space, a.Name.Local, a.Value, e, settings.PreserveDuplicateAttrs)
 			}
 			stack.push(e)
 		case xml.EndElement:
@@ -1223,17 +1236,20 @@ func (e *Element) addChild(t Token) {
 // prefix followed by a colon.
 func (e *Element) CreateAttr(key, value string) *Attr {
 	space, skey := spaceDecompose(key)
-	return e.createAttr(space, skey, value, e)
+	return e.createAttr(space, skey, value, e, false)
 }
 
 // createAttr is a helper function that creates attributes.
-func (e *Element) createAttr(space, key, value string, parent *Element) *Attr {
-	for i, a := range e.Attr {
-		if space == a.Space && key == a.Key {
-			e.Attr[i].Value = value
-			return &e.Attr[i]
+func (e *Element) createAttr(space, key, value string, parent *Element, preserveDups bool) *Attr {
+	if !preserveDups {
+		for i, a := range e.Attr {
+			if space == a.Space && key == a.Key {
+				e.Attr[i].Value = value
+				return &e.Attr[i]
+			}
 		}
 	}
+
 	a := Attr{
 		Space:   space,
 		Key:     key,
